package de.fh_zwickau.bumb_l_b.asa4od;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

/**
 * This class is an example application for a bflow* toolbox add-on. It shows
 * how an add-on can look like and demonstrates to print messages on the problems
 * view and adjust element's attributes.
 * 
 * @author Mathias Grunert, Florian Schoenfelder
 * 
 */
public class ASA4OD {

	/**
	 * Executes the analysis of given arguments.
	 * 
	 * @param args
	 * - arguments for execution
	 */
	public void execute(final String[] args) {

		if (args.length < 1) {
			printError("There is no path to a file given.");
			System.exit(1);
		}
		String path = args[0];

		int eventCounter, functionCounter = eventCounter = 0;

		try (BufferedReader br = new BufferedReader(new FileReader(path))) {
			String line;
			while ((line = br.readLine()) != null) {
				// [0]:type, [1]:id,[2]:name
				String[] pieces = line.split(":", 3);
				switch (pieces[0]) {
				case "event":
					printInfo("Event \"" + pieces[2] + "\" was marked. ",
							pieces[1]);
					printAttributeString("SET", pieces[1], "$fontstyle", "BOLD");
					eventCounter++;
					break;
				case "function":
					printInfo("Function \"" + pieces[2] + "\" was marked. ",
							pieces[1]);
					printAttributeString("SET", pieces[1], "$fontsize", "30");
					functionCounter++;
					break;
				case "edges":
					printInfo("The model contains " + pieces[1] + " edges.");
					break;
				}
			}
			if (eventCounter == 0 && functionCounter == 0) {
				printWarning("0 events or functions were marked.");
			}
		} catch (IOException e) {
			printError("An error occured while execution.");
		}
	}

	/**
	 * Prints a message of type info.
	 * 
	 * @param message
	 */
	private void printInfo(String message) {
		printInfo(message, "");
	}

	/**
	 * Prints a message of type warning.
	 * 
	 * @param message
	 */
	private void printWarning(String message) {
		printMessage("WARNING", message, "");
	}

	/**
	 * Print a message of type info connected to the given element.
	 * 
	 * @param message
	 * @param elementId
	 */
	private void printInfo(String message, String elementId) {
		printMessage("INFO", message, elementId);
	}

	/**
	 * Prints a message of type error.
	 * 
	 * @param message
	 */
	private void printError(String message) {
		printMessage("ERROR", message, "");
	}

	/**
	 * Prints a, for the bflow* problems-view formated, string with given type,
	 * message and element.
	 * 
	 * @param type
	 * - type of message
	 * @param message
	 * - message for the problems view
	 * @param elementId
	 * - the with the message connected element
	 */
	private void printMessage(String type, String message, String elementId) {
		printFormatString("MESSAGE", type, elementId, message, "");
	}

	/**
	 * Prints a string for attribute modification.
	 * 
	 * @param type
	 * @param elementId
	 * @param key
	 * @param value
	 */
	private void printAttributeString(String type, String elementId,
			String key, String value) {
		printFormatString("ATTRIBUTE", type, elementId, key, value);
	}

	/**
	 * Prints a for bflow* analysable string.
	 * 
	 * @param formatType
	 * @param type
	 * @param elementId
	 * @param key
	 * @param value
	 */
	private void printFormatString(String formatType, String type,
			String elementId, String key, String value) {
		System.out.println(String.format("addon:[%s][%s][%s][%s][%s]#FS#",
				formatType, type, elementId, key, value));

	}

	/**
	 * Starts the application.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		new ASA4OD().execute(args);
	}
}